//
//  SNSShareAuthorize.m
//  SNSShare
//
//  Created by 施 为 on 12-7-2.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "SNSShareAuthorize.h"
#import "SNSShareSDKGlobal.h"

#define kSNSShareAuthorizeURL           @"https://api.weibo.com/oauth2/authorize"
#define kSNSShareAccessTokenURL         @"https://api.weibo.com/oauth2/access_token"

@interface SNSShareAuthorize (private)

- (void) dismissModalViewController;
- (void) requestAccessTokenWithAuthorizeCode:(NSString*)code;
- (void) requestAccessTokenWithUserID:(NSString*)userID password:(NSString*)password;

@end

@implementation SNSShareAuthorize

@synthesize appKey = _appKey;
@synthesize appSecret = _appSecret;
@synthesize redirectURL = _redirectURL;
@synthesize request = _request;
@synthesize rootViewController = _rootViewController;
@synthesize delegate = _delegate;

#pragma mark - SNSShare Authorize Life Circle

- (void) dealloc
{
    [_appKey release], _appKey = nil;
    [_appSecret release], _appSecret = nil;
    
    [_redirectURL release], _redirectURL = nil;
    
    [_request setDelegate:nil];
    [_request disconnect];
    [_request release];
    
    _rootViewController = nil;
    _delegate = nil;
    
    [super dealloc];
}

- (id) initWithAppKey:(NSString *)theAppKey appSecret:(NSString *)theAppSecret
{
    if (self = [super init]) {
        self.appKey = theAppKey;
        self.appSecret = theAppSecret;
    }
    
    return self;
}

#pragma mark -
#pragma mark - SNSShareAuthorize private method

- (void) dismissModalViewController
{
    [_rootViewController dismissModalViewControllerAnimated:YES];
}

- (void) requestAccessTokenWithAuthorizeCode:(NSString *)code
{
    NSDictionary* params = [NSDictionary dictionaryWithObjectsAndKeys:_appKey,@"client_id",
                                                                    _appSecret,@"client_secret",
                                                                    @"authorization_code",@"grant_type",
                                                                    _redirectURL,@"redirect_url",
                                                                    code,@"code",
                                                                    nil];
    //如果request已经链接，先取消链接
    [_request disconnect];
    
    //发起注册，首先初始化注册参数
    _request = [SNSShareRequest requestWithURL:kSNSShareAccessTokenURL 
                                    httpMethod:@"POST" 
                                        params:params 
                                  postDataType:kSNSRequestPostDataTypeNormal 
                              httpHeaderFields:nil 
                                      delegate:self];
    
    [_request connect];
}

- (void) requestAccessTokenWithUserID:(NSString *)userID password:(NSString *)password
{
    NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:_appKey, @"client_id",
                                                                    _appSecret, @"client_secret",
                                                                    @"password", @"grant_type",
                                                                    _redirectURL, @"redirect_url",
                                                                    userID, @"username",
                                                                    password, @"password", 
                                                                    nil];
    [_request disconnect];
    
    _request = [SNSShareRequest requestWithURL:kSNSShareAccessTokenURL 
                                    httpMethod:@"POST" 
                                        params:params 
                                  postDataType:kSNSRequestPostDataTypeNormal 
                              httpHeaderFields:nil 
                                      delegate:self];
    
    [_request connect];
}       

#pragma mark -
#pragma mark - SNSShareAuthorize Public Methods

/*!
 启动微博认证过程
 */
- (void) startAuthorized
{
    NSDictionary* params = [NSDictionary dictionaryWithObjectsAndKeys:_appKey,@"client_id",
                                                                    @"code",@"response_type",
                                                                    _redirectURL,@"redirect_url",
                                                                    @"mobile",@"display",nil];
    NSString* urlString = [SNSShareRequest serializeURL:kSNSShareAuthorizeURL 
                                                 params:params 
                                             httpMehtod:@"GET"];
    
    SNSShareAuthorizeWebView* webView = [[SNSShareAuthorizeWebView alloc] init];
    [webView setDelegate:self];
    [webView loadRequestWithURL:[NSURL URLWithString:urlString]];
    
    [webView show:YES];
    [webView release];
}

- (void) startAuthorizedUsingUserID:(NSString *)userID password:(NSString *)password
{
    [self requestAccessTokenWithUserID:userID password:password];
}

#pragma mark -
#pragma mark - SNSShareAuthorizeWebViewDelegate Method
- (void)authorizeWebView:(SNSShareAuthorizeWebView *)webView didReceiveAuthorizeCode:(NSString *)code
{
    [webView hide:YES];
    
    if (![code isEqualToString:@"21330"]) {
        [self requestAccessTokenWithAuthorizeCode:code];
    }
}

#pragma mark -
#pragma mark - SNSShareRequestDelegate Methods
- (void) request:(SNSShareRequest*)theRequest didFinishLoadingWithResult:(id)result
{
    BOOL success = NO;
    
    if ([result isKindOfClass:[NSDictionary class]]) {
        NSDictionary* dict = (NSDictionary*)result;
        
        NSString*   token = [dict objectForKey:@"access_token"];
        NSString*   userID = [dict objectForKey:@"uid"];
        NSInteger   seconds = [[dict objectForKey:@"expires_in"] intValue];
        
        success = token && userID;
        
        if (success && [_delegate respondsToSelector:@selector(authorize:didSucceedWithAccessToken:userID:expiresIn:)]) {
            [_delegate authorize:self didSucceedWithAccessToken:token userID:userID expiresIn:seconds];
        }
    }
    
    //收到但是不还是不成功
    if (!success && [_delegate respondsToSelector:@selector(authorize:didFailedWithError:)]) {
        NSError* error = [NSError errorWithDomain:kSNSShareSDKErrorDomain 
                                             code:kSNSShareErrorCodeSDK 
                                         userInfo:[NSDictionary dictionaryWithObject:[NSString stringWithFormat:@"%d",kSNSShareSDKErrorCodeAuthorizeError] 
                                                                              forKey:kSNSShareSDKErrorCodeKey]]; 
        [_delegate authorize:self didFailedWithError:error];
    }
}

- (void) request:(SNSShareRequest *)request didFailWithError:(NSError *)error
{
    if([_delegate respondsToSelector:@selector(authorize:didFailedWithError:)])
    {
        [_delegate authorize:self didFailedWithError:error];
    }
}

@end
