//
//  SNSShareAuthorizeWebView.m
//  SNSShare
//
//  Created by 施 为 on 12-7-4.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "SNSShareAuthorizeWebView.h"
#import <QuartzCore/QuartzCore.h>

@interface SNSShareAuthorizeWebView (Private)

- (void) bounceOutAnimationStopped;
- (void) bounceInAnimationStopped;
- (void) bounceNormalAnimationStopped;
- (void) allAnimationsStopped;

- (UIInterfaceOrientation) currentOrientation;
- (void) sizeToFitOrientation:(UIInterfaceOrientation)orientation;
- (CGAffineTransform)transformForOrientation:(UIInterfaceOrientation)orientation;

- (void)addObservers;
- (void)removeObservers;

@end

@implementation SNSShareAuthorizeWebView

@synthesize delegate = _delegate;

#pragma mark - SNSShareAuthorizeWebView Life Circle

- (id)init
{
    if (self = [super initWithFrame:CGRectMake(0, 0, 320, 480)]) {
        //background settings
        [self setBackgroundColor:[UIColor clearColor]];
        [self setAutoresizingMask:UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight];
        
        // add the panel view
        _panelView = [[UIView alloc] initWithFrame:CGRectMake(10,30,300,440)];
        [_panelView setBackgroundColor:[UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:0.55]];
        [[_panelView layer] setMasksToBounds:NO];   //Very important
        [[_panelView layer] setCornerRadius:10.0];
        [self addSubview:_panelView];
        
        //add the container view
        _containerView = [[UIView alloc] initWithFrame:CGRectMake(10, 10, 280, 420)];
        [[_containerView layer] setBorderColor:[UIColor colorWithRed:0. green:0. blue:0. alpha:0.].CGColor];
        [[_containerView layer] setBorderWidth:1.0];
        
        //add the web view
        _webView = [[UIWebView alloc] initWithFrame:CGRectMake(0,0, 280, 390)];
        [_webView setDelegate:self];
        [_containerView addSubview:_webView];
        
        [_panelView addSubview:_containerView];
        
        _indicatorView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
        [_indicatorView setCenter:CGPointMake(160, 240)];
        [self addSubview:_indicatorView];
    }
    return self;
}

- (void) dealloc
{
    [_panelView release], _panelView = nil;
    [_containerView release], _containerView = nil;
    [_webView release], _webView = nil;
    [_indicatorView release], _indicatorView = nil;
    [super dealloc];
}

#pragma mark Action

- (void) onCloseButtonTouched:(id)sender
{
    [self hide:YES];
}

#pragma mark Orientations
- (UIInterfaceOrientation) currentOrientation
{
    return [UIApplication sharedApplication].statusBarOrientation;
}

- (void)sizeToFitOrientation:(UIInterfaceOrientation)orientation
{
    [self setTransform:CGAffineTransformIdentity];
    
    if (UIInterfaceOrientationIsLandscape(orientation)) {
        [self   setFrame:CGRectMake(0, 0, 480, 320)];
        [_panelView setFrame:CGRectMake(10, 30, 460, 260)];
        [_containerView setFrame:CGRectMake(10, 10, 440, 260)];
        [_webView setFrame:CGRectMake(0, 0, 440, 260)];
        [_indicatorView setCenter:CGPointMake(240, 160)];
    }
    else {
        [self   setFrame:CGRectMake(0, 0, 320, 480)];
        [_panelView setFrame:CGRectMake(10, 30, 300, 420)];
        [_containerView setFrame:CGRectMake(10, 10, 280, 400)];
        [_webView setFrame:CGRectMake(0, 0, 280, 400)];
        [_indicatorView setCenter:CGPointMake(160, 240)];
    }
    
    [self setCenter:CGPointMake(160, 240)];
    [self setTransform:[self transformForOrientation:orientation]];
    
    _previousOrientation = orientation;
}

- (CGAffineTransform)transformForOrientation:(UIInterfaceOrientation)orientation
{
    if (orientation == UIInterfaceOrientationLandscapeLeft) {
        return CGAffineTransformMakeRotation(- M_PI_2);
    }
    else if (orientation == UIInterfaceOrientationPortraitUpsideDown) {
        return CGAffineTransformMakeRotation(- M_PI);
    }
    else if (orientation == UIInterfaceOrientationLandscapeRight) {
        return CGAffineTransformMakeRotation(M_PI_2);
    }
    else {
        return CGAffineTransformIdentity;
    }
}

- (BOOL)shouldRotateToOrientation:(UIInterfaceOrientation)orientation 
{
    if (orientation == _previousOrientation) {
        return NO;
    }
    else {
        return orientation == UIInterfaceOrientationLandscapeLeft
        || orientation == UIInterfaceOrientationLandscapeRight
        || orientation == UIInterfaceOrientationPortraitUpsideDown
        || orientation == UIInterfaceOrientationPortrait;
    }
    
    return YES;
}

#pragma mark - Observers

- (void) addObservers
{
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(deviceOrientationDidChange:)
                                                 name:@"UIDeviceOrientationDidChangeNotifation" 
                                               object:nil];
}

- (void) removeObservers
{
    [[NSNotificationCenter defaultCenter] removeObserver:self 
                                                    name:@"UIDeviceOrientationDidChangeNotifation" 
                                                  object:nil];
}

#pragma mark - Animations
- (void) bounceOutAnimationStopped
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.13];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(bounceInAnimationStopped)];
    [_panelView setAlpha:0.8];
    [_panelView setTransform:CGAffineTransformScale(CGAffineTransformIdentity, .9, .9)];
    [UIView commitAnimations];
}

- (void) bounceInAnimationStopped
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:.13];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(bounceNormalAnimationStopped)];
    [_panelView setAlpha:1.0];
    [_panelView setTransform:CGAffineTransformScale(CGAffineTransformIdentity, 1.0, 1.0)];
    [UIView commitAnimations];
}

- (void) bounceNormalAnimationStopped
{
    [self allAnimationsStopped];
}

- (void) allAnimationsStopped
{
    //nothine shall be done here 
    //why?
}

#pragma mark - Dismiss

- (void) hideAndCleanUp
{
    [self   removeObservers];
    [self removeFromSuperview];
}

#pragma mark - SNSShareAuthorizeWebView
- (void)loadRequestWithURL:(NSURL *)url
{
    NSURLRequest* request = [NSURLRequest requestWithURL:url cachePolicy:NSURLRequestReloadIgnoringCacheData timeoutInterval:60.0];
    
    [_webView loadRequest:request];
}

#pragma mark - SNSShareAuthorizeWebView Public Methods
- (void) show:(BOOL)animated
{
    [self sizeToFitOrientation:[self currentOrientation]];
    
    UIWindow* window = [UIApplication sharedApplication].keyWindow;
    
    if (!window) {
        window = [[UIApplication sharedApplication].windows objectAtIndex:0];
    }
    [window addSubview:self];
    
    if (animated) {
        [_panelView setAlpha:0];
        CGAffineTransform transform = CGAffineTransformIdentity;
        [_panelView setTransform:CGAffineTransformScale(transform, 0.3, 0.3)];
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDuration:0.2];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(bounceOutAnimationStopped)];
        [_panelView setAlpha:0.5];
        [_panelView setTransform:CGAffineTransformScale(transform, 1.1, 1.1)];
        [UIView commitAnimations];
    }
    else {
        [self allAnimationsStopped];
    }
    
    [self addObservers];
}

- (void) hide:(BOOL)animated
{
    if (animated) {
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDuration:0.3];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(hideAndCleanUp:)];
        [self setAlpha:0];
        [UIView commitAnimations];
    }
    [self  hideAndCleanUp];
}

#pragma mark - UIDeviceOrientationDidChangeNotifation Methods

- (void) deviceOrientationDidChange:(id)object
{
    UIInterfaceOrientation orientation = [self currentOrientation];
    if ([self shouldRotateToOrientation:orientation]) {
        NSTimeInterval duration = [UIApplication sharedApplication].statusBarOrientationAnimationDuration;
        
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDuration:duration];
        [UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
        [self sizeToFitOrientation:orientation];
        [UIView commitAnimations];
    }
}

#pragma mark - UIWebViewDelegation Method

- (void) webViewDidStartLoad:(UIWebView *)webView
{
    [_indicatorView startAnimating];
}

- (void) webViewDidFinishLoad:(UIWebView *)webView
{
    [_indicatorView stopAnimating];
}

- (void) webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
    [_indicatorView stopAnimating];
}

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    NSRange range = [request.URL.absoluteString rangeOfString:@"code="];
    
    if (range.location != NSNotFound) {
        NSString* code = [request.URL.absoluteString substringFromIndex:range.location + range.length];
        
        if ([_delegate respondsToSelector:@selector(authorizeWebView:didReceiveAuthorizeCode:)]) {
            [_delegate authorizeWebView:self didReceiveAuthorizeCode:code];
        }
    }
    
    return YES;
}

@end
