//
//  SNSShareEngine.h
//  SNSShare
//
//  Created by 施 为 on 12-7-2.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "SNSShareAuthorize.h"
#import "SNSShareRequest.h"

@class SNSShareEngine;

@protocol SNSShareEngineDelegate <NSObject>

@optional

//If you try to log in with logIn or logInUsingUserID method,
- (void) engineAlreaddyLoggedIn:(SNSShareEngine*) engine;

//Log in successfully
- (void) engineLogInSucceed:(SNSShareEngine*) engine;

//Filed to log in.
//Possible reasons are:
//1.Either username or password is wrong
//2.Your app has not been authorized by sina yet.
- (void) engine:(SNSShareEngine*)engine didFailToLogInWithError:(NSError*)error;

//log out successfully
- (void) engineLogOutSucceed:(SNSShareEngine*)engine;

//When you use the SNSShareRequest Methods,
//you may receive the following four callback.
- (void) engineNotAuthorized:(SNSShareEngine*)engine;
- (void) engineAuthorizedExpired:(SNSShareEngine*)engien;

- (void) engine:(SNSShareEngine*)engine requestDidFailWithError:(NSError*)error;
- (void) engine:(SNSShareEngine*)engine requestDidSucceedWithRusult:(id)result;

@end

@interface SNSShareEngine : NSObject<
                            SNSShareAuthorizeDelegate,
                            SNSShareRequestDelegate
>{
    NSString*       _appKey;
    NSString*       _appSecret;
    
    NSString*       _userID;
    NSString*       _accessToken;
    NSTimeInterval  _expireTime;
    
    NSString*       _redirectURL;
    
    //Determine whether user must log out first before log in other account.
    BOOL            _isUserExclusive;
    
    SNSShareAuthorize*  _auth;
    SNSShareRequest*    _request;
    
    id<SNSShareEngineDelegate> _delegate;

    UIViewController* _rootViewController;
}

@property (nonatomic, retain) NSString* appKey;
@property (nonatomic, retain) NSString* appSecret;
@property (nonatomic, retain) NSString* userID;
@property (nonatomic, retain) NSString* accessToken;
@property (nonatomic, assign) NSTimeInterval expireTime;
@property (nonatomic, retain) NSString* redirectURL;
@property (nonatomic, assign) BOOL      isUserExclusive;
@property (nonatomic, retain) SNSShareRequest* request;
@property (nonatomic, retain) SNSShareAuthorize* auth;
@property (nonatomic, assign) id<SNSShareEngineDelegate> delegate;
@property (nonatomic, assign) UIViewController* rootViewController;

//Initilize an instance with AppKey and AppSecret you have for your client.
- (id)initWithAppKey:(NSString*) theAppKey appSecret:(NSString*) theAppSecret;

//Log in Using OAuth Web Authorization.
//If succeed, engineLogInSucceed will be called.
- (void) logIn;

// Log in using OAuth Client authorization.
// If succeed, engineDidLogIn will be called.
- (void)logInUsingUserID:(NSString *)theUserID password:(NSString *)thePassword;

//log out
//If succeed, engineLogOutSucceed will be called.
- (void) logOut;

//Check if user has logged in,or the authorization is expired
- (BOOL) isLoggedIn;
- (BOOL) isAuthorizeExpired;

/*!
 @method        loadRequestWithMethodName:httpMethod:param:pastDataType:httpHeaderFields:
 @discussion    
 
 @param         methodName      the name of method to load request
                httpMethod      "GET" or "POST"
                param           A dictionary that contains your request parameters
                postDataType    "GET" for kSNSRequestPostDataTypeNone,"POST" for kSNSRequestPostDataTypeNormal or kSNSRequestPostDataTypeMultipart
 @httpHeaderFields: A dictionary that contains HTTP header information.
 */
- (void) loadRequestWithMethodName:(NSString*)methodName
                        httpMethod:(NSString*)httpMethod
                            params:(NSDictionary*)params
                      postDataType:(SNSRequestPostDataType)postDataType
                  httpHeaderFields:(NSDictionary*)httpHeaderFields;

//Send a WeiBo, to which you can attach an image.
- (void) sendWeiBoWithText:(NSString*) text image:(UIImage*)image;

@end
