//
//  SNSShareEngine.m
//  SNSShare
//
//  Created by 施 为 on 12-7-2.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "SNSShareEngine.h"
#import "SFHFKeychainUtils.h"
#import "SNSShareSDKGlobal.h"

#define kSNSShareURLSchemePrefix        @"SNSShare_"
#define KSNSShareKeyChainNameSuffix     @"_WeiBoServiceName"

#define kSNSShareKeychainUserID         @"SNSShareUserID"
#define kSNSShareKeychainAccessToken    @"SNSShareAccessToken"
#define kSNSShareKeychainExpireTime     @"SNSShareExpireTime"


@interface SNSShareEngine(private)

- (NSString*) urlSchemeString;

- (void)saveAuthorizeDataToKeyChain;
- (void)readAuthorizeDataFromKeyChain;
- (void)deleteAuthorizeDataInKeyChain;

@end

@implementation SNSShareEngine

@synthesize appKey = _appKey;
@synthesize appSecret = _appSecret;
@synthesize userID = _userID;
@synthesize accessToken = _accessToken;
@synthesize expireTime = _expireTime;
@synthesize redirectURL = _redirectURL;
@synthesize isUserExclusive = _isUserExclusive;
@synthesize request = _request;
@synthesize auth = _auth;
@synthesize delegate = _delegate;
@synthesize rootViewController = _rootViewController;

#pragma mark - WBEngine Life Circle
- (void) dealloc
{
    [_appKey release], _appKey = nil;
    [_appSecret release], _appSecret = nil;
    
    [_userID release], _userID = nil;
    [_accessToken release], _accessToken = nil;
    
    [_redirectURL release], _redirectURL = nil;
    
    [_request setDelegate:nil];
    [_request disconnect];
    [_request release], _request = nil;
    
    [_auth setDelegate:nil];
    [_auth release], _auth = nil;
    
    _delegate = nil;
    
    _rootViewController = nil;
    
    [super dealloc];
}

- (id) initWithAppKey:(NSString *)theAppKey appSecret:(NSString *)theAppSecret
{
    if (self = [super init]) {
        self.appKey = theAppKey;
        self.appSecret = theAppSecret;
        
        _isUserExclusive = NO;
        
        [self readAuthorizeDataFromKeyChain];
    }
    
    return self;
}

#pragma mark - SNSShareEngine Private Method

- (NSString*) urlSchemeString
{
    return [NSString stringWithFormat:@"%@%@", kSNSShareURLSchemePrefix, _appKey];
}

- (void) saveAuthorizeDataToKeyChain
{
    NSString* serviceName = [[self urlSchemeString] stringByAppendingString:KSNSShareKeyChainNameSuffix];
    [SFHFKeychainUtils storeUsername:kSNSShareKeychainUserID andPassword:_userID forServiceName:serviceName updateExisting:YES error:nil];
    [SFHFKeychainUtils storeUsername:kSNSShareKeychainAccessToken andPassword:_accessToken forServiceName:serviceName updateExisting:YES error:nil];
    [SFHFKeychainUtils storeUsername:kSNSShareKeychainExpireTime andPassword:[NSString stringWithFormat:@"%lf",_expireTime] forServiceName:serviceName updateExisting:YES error:nil];
}

- (void) readAuthorizeDataFromKeyChain
{
    NSString* serviceName = [[self urlSchemeString] stringByAppendingString:KSNSShareKeyChainNameSuffix];
    self.userID = [SFHFKeychainUtils getPasswordForUsername:kSNSShareKeychainUserID andServiceName:serviceName error:nil];
    self.accessToken = [SFHFKeychainUtils getPasswordForUsername:kSNSShareKeychainAccessToken andServiceName:serviceName error:nil];
    self.expireTime = [[SFHFKeychainUtils getPasswordForUsername:kSNSShareKeychainExpireTime andServiceName:serviceName error:nil] doubleValue];
}

- (void) deleteAuthorizeDataInKeyChain
{
    self.userID = nil;
    self.accessToken = nil;
    self.expireTime = 0;
    
    NSString* serviceName = [[self urlSchemeString] stringByAppendingString:KSNSShareKeyChainNameSuffix];
    [SFHFKeychainUtils deleteItemForUsername:kSNSShareKeychainUserID andServiceName:serviceName error:nil];
    [SFHFKeychainUtils deleteItemForUsername:kSNSShareKeychainAccessToken andServiceName:serviceName error:nil];
    [SFHFKeychainUtils deleteItemForUsername:kSNSShareKeychainExpireTime andServiceName:serviceName error:nil];
}

#pragma mark - WBEngine Public Methods

#pragma mark Authorization
- (void) logIn
{
    //如果用户已经登陆，发起delegate告诉EngineDelegate已经注册；而不支持多账号登陆情况下再次登陆直接返回退出
    if ([self isLoggedIn]) {
        if ([_delegate respondsToSelector:@selector(engineAlreaddyLoggedIn:)]) {
            [_delegate engineAlreaddyLoggedIn:self];
        }
        if (_isUserExclusive) {
            return;
        }
    }
    
    //初始化SNSShareAuthorize，发起认证的类，并设置相应代理
    SNSShareAuthorize* auth = [[SNSShareAuthorize alloc] initWithAppKey:_appKey appSecret:_appSecret];
    [auth setRootViewController:_rootViewController];
    [auth setDelegate:self];
    self.auth = auth;
    [auth release];
    
    //确定回调地址
    if ([_redirectURL length] > 0) {
        [_auth setRedirectURL:_redirectURL];
    }
    else {
        [_auth setRedirectURL:@"http://"];
    }
    
    //启动微博OAuth2.0的认证过程
    [_auth startAuthorized];
}

- (void) logInUsingUserID:(NSString *)theUserID password:(NSString *)thePassword
{
    self.userID = theUserID;
    
    if ([self isLoggedIn]) {
        if ([_delegate respondsToSelector:@selector(engineAlreaddyLoggedIn:)]) {
            [_delegate engineAlreaddyLoggedIn:self];
        }
        if (_isUserExclusive) {
            return;
        }
    }
    
    SNSShareAuthorize* auth = [[SNSShareAuthorize alloc] initWithAppKey:_appKey appSecret:_appSecret];
    [auth setRootViewController:_rootViewController];
    [auth setDelegate:self];
    self.auth = auth;
    [auth release];
    
    if ([_redirectURL length] > 0) {
        [_auth setRedirectURL:_redirectURL];
    }
    else {
        [_auth setRedirectURL:@"http://"];
    }
    
    [_auth startAuthorizedUsingUserID:theUserID password:thePassword];
}

- (void) logOut
{
    [self deleteAuthorizeDataInKeyChain];
    
    if ([_delegate respondsToSelector:@selector(engineLogOutSucceed:)]) {
        [_delegate engineLogOutSucceed:self];
    }
}

- (BOOL) isLoggedIn
{
    //如果有urserID,accessToken,expireTime则认为未过期 
    return _userID && _accessToken && (_expireTime > 0);
}

- (BOOL) isAuthorizeExpired
{
    if ([[NSData data] timeIntervalSince1970] > _expireTime) {
        
        //force to log out, 这里只是清除keychain保存信息，不用delegate发出成功登出的消息
        [self deleteAuthorizeDataInKeyChain];
        return YES;
    }
    
    return NO;
}

#pragma mark Request
- (void) loadRequestWithMethodName:(NSString*)methodName
                        httpMethod:(NSString*)httpMethod
                            params:(NSDictionary*)params
                      postDataType:(SNSRequestPostDataType)postDataType
                  httpHeaderFields:(NSDictionary*)httpHeaderFields
{
    // Step 1.
    // Check if the user has been logged in.
    if (![self isLoggedIn]) {
        if ([_delegate respondsToSelector:@selector(engineNotAuthorized:)]) {
            [_delegate engineNotAuthorized:self];
        }
        return;
    }
    
    // Step 2.
    // Check if the access token is expired.
    if ([self isAuthorizeExpired]) {
        if ([_delegate respondsToSelector:@selector(engineAuthorizedExpired:)]) {
            [_delegate engineAuthorizedExpired:self];
        }
    }
    
    [_request disconnect];

    _request = [SNSShareRequest requestWithAccessToken:_accessToken
                                                   url:[NSString stringWithFormat:@"%@%@",kSNSShareSDKAPIDomain,methodName] 
                                            httpMethod:httpMethod
                                                params:params 
                                          postDataType:postDataType 
                                      httpHeaderFields:httpHeaderFields 
                                              delegate:self];
    [_request connect];
}

#pragma mark -
#pragma mark - SNSShareAuthorize Delegate Method

/*!
 认证成功，从Authorize Class 回到engine类。1，告诉用户认证成功；2，保存认证过程产生的信息
 */
- (void) authorize:(SNSShareAuthorize *)authorize didFailedWithError:(NSError *)error
{
    if ([_delegate respondsToSelector:@selector(engine:didFailToLogInWithError:)]) {
        [_delegate engine:self didFailToLogInWithError:error];
    }
}

- (void) authorize:(SNSShareAuthorize *)authorize didSucceedWithAccessToken:(NSString *)accessToken userID:(NSString *)userID expiresIn:(NSInteger)seconds
{
    self.accessToken = accessToken;
    self.userID = userID;
    self.expireTime = [[NSDate date] timeIntervalSince1970] + seconds;
    
    [self saveAuthorizeDataToKeyChain];
    
    if ([_delegate respondsToSelector:@selector(engineLogInSucceed:)]) {
        [_delegate engineLogInSucceed:self];
    }
}

#pragma mark -
#pragma mark - SNSShareRequest Delegate Method

/*!
 这里调用SNSShareEngineDelegate的相应delegate method,反馈认证失败，反馈出来的信息给最终用户使用
 */
- (void) request:(SNSShareRequest *)request didFailWithError:(NSError *)error
{
    if ([_delegate respondsToSelector:@selector(engine:requestDidFailWithError:)]) {
        [_delegate engine:self requestDidFailWithError:error];
    }
}

- (void) request:(SNSShareRequest *)request didFinishLoadingWithResult:(id)result
{
    if ([_delegate respondsToSelector:@selector(engine:requestDidSucceedWithRusult:)]) {
        [_delegate engine:self requestDidSucceedWithRusult:result];
    }
}

@end
